package org.apache.bcel.generic;

import java.io.DataOutputStream;
import java.io.IOException;
import org.apache.bcel.util.ByteSequence;

public abstract class Select extends BranchInstruction implements VariableLengthInstruction, StackConsumer, StackProducer {
   /** @deprecated */
   @Deprecated
   protected int[] match;
   /** @deprecated */
   @Deprecated
   protected int[] indices;
   /** @deprecated */
   @Deprecated
   protected InstructionHandle[] targets;
   /** @deprecated */
   @Deprecated
   protected int fixed_length;
   /** @deprecated */
   @Deprecated
   protected int match_length;
   /** @deprecated */
   @Deprecated
   protected int padding;

   Select() {
   }

   Select(short opcode, int[] match, InstructionHandle[] targets, InstructionHandle defaultTarget) {
      super(opcode, (InstructionHandle)null);
      this.match = match;
      this.targets = targets;
      this.setTarget(defaultTarget);
      InstructionHandle[] var5 = targets;
      int var6 = targets.length;

      for(int var7 = 0; var7 < var6; ++var7) {
         InstructionHandle target2 = var5[var7];
         notifyTarget((InstructionHandle)null, target2, this);
      }

      if ((this.match_length = match.length) != targets.length) {
         throw new ClassGenException("Match and target array have not the same length: Match length: " + match.length + " Target length: " + targets.length);
      } else {
         this.indices = new int[this.match_length];
      }
   }

   protected Object clone() throws CloneNotSupportedException {
      Select copy = (Select)super.clone();
      copy.match = (int[])this.match.clone();
      copy.indices = (int[])this.indices.clone();
      copy.targets = (InstructionHandle[])this.targets.clone();
      return copy;
   }

   public boolean containsTarget(InstructionHandle ih) {
      if (super.getTarget() == ih) {
         return true;
      } else {
         InstructionHandle[] var2 = this.targets;
         int var3 = var2.length;

         for(int var4 = 0; var4 < var3; ++var4) {
            InstructionHandle target2 = var2[var4];
            if (target2 == ih) {
               return true;
            }
         }

         return false;
      }
   }

   void dispose() {
      super.dispose();
      InstructionHandle[] var1 = this.targets;
      int var2 = var1.length;

      for(int var3 = 0; var3 < var2; ++var3) {
         InstructionHandle target2 = var1[var3];
         target2.removeTargeter(this);
      }

   }

   public void dump(DataOutputStream out) throws IOException {
      out.writeByte(super.getOpcode());

      for(int i = 0; i < this.padding; ++i) {
         out.writeByte(0);
      }

      super.setIndex(this.getTargetOffset());
      out.writeInt(super.getIndex());
   }

   final int getFixedLength() {
      return this.fixed_length;
   }

   public int[] getIndices() {
      return this.indices;
   }

   final int getIndices(int index) {
      return this.indices[index];
   }

   final int getMatch(int index) {
      return this.match[index];
   }

   final int getMatchLength() {
      return this.match_length;
   }

   public int[] getMatchs() {
      return this.match;
   }

   final int getPadding() {
      return this.padding;
   }

   final InstructionHandle getTarget(int index) {
      return this.targets[index];
   }

   public InstructionHandle[] getTargets() {
      return this.targets;
   }

   protected void initFromFile(ByteSequence bytes, boolean wide) throws IOException {
      this.padding = (4 - bytes.getIndex() % 4) % 4;

      for(int i = 0; i < this.padding; ++i) {
         bytes.readByte();
      }

      super.setIndex(bytes.readInt());
   }

   final void setFixedLength(int fixedLength) {
      this.fixed_length = fixedLength;
   }

   final int setIndices(int i, int value) {
      this.indices[i] = value;
      return value;
   }

   final void setIndices(int[] array) {
      this.indices = array;
   }

   final void setMatch(int index, int value) {
      this.match[index] = value;
   }

   final int setMatchLength(int matchLength) {
      this.match_length = matchLength;
      return matchLength;
   }

   final void setMatches(int[] array) {
      this.match = array;
   }

   public void setTarget(int i, InstructionHandle target) {
      notifyTarget(this.targets[i], target, this);
      this.targets[i] = target;
   }

   final void setTargets(InstructionHandle[] array) {
      this.targets = array;
   }

   public String toString(boolean verbose) {
      StringBuilder buf = new StringBuilder(super.toString(verbose));
      if (verbose) {
         for(int i = 0; i < this.match_length; ++i) {
            String s = "null";
            if (this.targets[i] != null) {
               s = this.targets[i].getInstruction().toString();
            }

            buf.append("(").append(this.match[i]).append(", ").append(s).append(" = {").append(this.indices[i]).append("})");
         }
      } else {
         buf.append(" ...");
      }

      return buf.toString();
   }

   protected int updatePosition(int offset, int maxOffset) {
      this.setPosition(this.getPosition() + offset);
      short oldLength = (short)super.getLength();
      this.padding = (4 - (this.getPosition() + 1) % 4) % 4;
      super.setLength((short)(this.fixed_length + this.padding));
      return super.getLength() - oldLength;
   }

   public void updateTarget(InstructionHandle oldIh, InstructionHandle newIh) {
      boolean targeted = false;
      if (super.getTarget() == oldIh) {
         targeted = true;
         this.setTarget(newIh);
      }

      for(int i = 0; i < this.targets.length; ++i) {
         if (this.targets[i] == oldIh) {
            targeted = true;
            this.setTarget(i, newIh);
         }
      }

      if (!targeted) {
         throw new ClassGenException("Not targeting " + oldIh);
      }
   }
}
